@@ -237,7 +237,7 @@ class TestResult(unittest.TestResult):
 
     def stopTest(self, test):
         # NOTE: In Python 3.12.1 skipped tests may not call startTest()
-        if self._tags is not None:
+        if self._tags is not None and self._tags.parent is not None:
             self._tags = self._tags.parent
         super().stopTest(test)
 
@@ -565,9 +565,20 @@ class StreamResultRouter(StreamResult):
             sink.stopTestRun()
         self._in_run = False
 
-    def status(self, **kwargs):
-        route_code = kwargs.get("route_code", None)
-        test_id = kwargs.get("test_id", None)
+    def status(
+        self,
+        test_id=None,
+        test_status=None,
+        test_tags=None,
+        runnable=True,
+        file_name=None,
+        file_bytes=None,
+        eof=False,
+        mime_type=None,
+        route_code=None,
+        timestamp=None,
+    ):
+        # route_code and test_id are already available as parameters
         if route_code is not None:
             prefix = route_code.split("/")[0]
         else:
@@ -578,12 +589,23 @@ class StreamResultRouter(StreamResult):
                 route_code = route_code[len(prefix) + 1 :]
                 if not route_code:
                     route_code = None
-                kwargs["route_code"] = route_code
+                # Update route_code for forwarding
         elif test_id in self._test_ids:
             target = self._test_ids[test_id]
         else:
             target = self.fallback
-        target.status(**kwargs)
+        target.status(
+            test_id=test_id,
+            test_status=test_status,
+            test_tags=test_tags,
+            runnable=runnable,
+            file_name=file_name,
+            file_bytes=file_bytes,
+            eof=eof,
+            mime_type=mime_type,
+            route_code=route_code,
+            timestamp=timestamp,
+        )
 
     def add_rule(self, sink, policy, do_start_stop_run=False, **policy_args):
         """Add a rule to route events to sink when they match a given policy.
@@ -735,7 +757,7 @@ class _TestRecord:
             case = self
         else:
             content_type = _make_content_type(mime_type)
-            content_bytes = []
+            content_bytes: list[bytes] = []
             case = self.transform(
                 ["details", file_name], Content(content_type, lambda: content_bytes)
             )
@@ -753,6 +775,7 @@ class _TestRecord:
         if PlaceHolder is None:
             from testtools.testcase import PlaceHolder
         outcome = _status_map[self.status]
+        assert PlaceHolder is not None
         return PlaceHolder(
             self.id,
             outcome=outcome,
@@ -1186,12 +1209,21 @@ class MultiTestResult(TestResult):
 class TextTestResult(TestResult):
     """A TestResult which outputs activity to a text stream."""
 
-    def __init__(self, stream, failfast=False, tb_locals=False):
-        """Construct a TextTestResult writing to stream."""
+    def __init__(self, stream, failfast=False, tb_locals=False, verbosity=1):
+        """Construct a TextTestResult writing to stream.
+
+        :param stream: A file-like object to write results to.
+        :param failfast: Stop after the first failure.
+        :param tb_locals: If True include local variables in tracebacks.
+        :param verbosity: Verbosity level. 0 for quiet, 1 for normal (dots, default),
+            2 for verbose (test names).
+        """
         super().__init__(failfast=failfast, tb_locals=tb_locals)
         self.stream = stream
         self.sep1 = "=" * 70 + "\n"
         self.sep2 = "-" * 70 + "\n"
+        self.verbosity = verbosity
+        self._progress_printed = False
 
     def _delta_to_float(self, a_timedelta, precision):
         # This calls ceiling to ensure that the most pessimistic view of time
@@ -1218,6 +1250,67 @@ class TextTestResult(TestResult):
             self.stream.write(self.sep2)
             self.stream.write(output)
 
+    def startTest(self, test):
+        super().startTest(test)
+        if self.verbosity >= 2:
+            self.stream.write(f"{test.id()} ... ")
+            self.stream.flush()
+
+    def addSuccess(self, test, details=None):
+        super().addSuccess(test, details=details)
+        if self.verbosity == 1:
+            self.stream.write(".")
+            self.stream.flush()
+            self._progress_printed = True
+        elif self.verbosity >= 2:
+            self.stream.write("ok\n")
+            self.stream.flush()
+
+    def addError(self, test, err=None, details=None):
+        super().addError(test, err=err, details=details)
+        if self.verbosity == 1:
+            self.stream.write("E")
+            self.stream.flush()
+        elif self.verbosity >= 2:
+            self.stream.write("ERROR\n")
+            self.stream.flush()
+
+    def addFailure(self, test, err=None, details=None):
+        super().addFailure(test, err=err, details=details)
+        if self.verbosity == 1:
+            self.stream.write("F")
+            self.stream.flush()
+        elif self.verbosity >= 2:
+            self.stream.write("FAIL\n")
+            self.stream.flush()
+
+    def addSkip(self, test, reason=None, details=None):
+        super().addSkip(test, reason=reason, details=details)
+        if self.verbosity == 1:
+            self.stream.write("s")
+            self.stream.flush()
+        elif self.verbosity >= 2:
+            self.stream.write(f"skipped {reason!r}\n")
+            self.stream.flush()
+
+    def addExpectedFailure(self, test, err=None, details=None):
+        super().addExpectedFailure(test, err=err, details=details)
+        if self.verbosity == 1:
+            self.stream.write("x")
+            self.stream.flush()
+        elif self.verbosity >= 2:
+            self.stream.write("expected failure\n")
+            self.stream.flush()
+
+    def addUnexpectedSuccess(self, test, details=None):
+        super().addUnexpectedSuccess(test, details=details)
+        if self.verbosity == 1:
+            self.stream.write("u")
+            self.stream.flush()
+        elif self.verbosity >= 2:
+            self.stream.write("unexpected success\n")
+            self.stream.flush()
+
     def startTestRun(self):
         super().startTestRun()
         self.__start = self._now()
@@ -1235,8 +1328,14 @@ class TextTestResult(TestResult):
             self.stream.write(
                 f"{self.sep1}UNEXPECTED SUCCESS: {test.id()}\n{self.sep2}"
             )
+        # Add newline(s) before summary
+        # If we printed progress indicators (dots), add extra newline
+        if self._progress_printed:
+            self.stream.write("\n\n")
+        else:
+            self.stream.write("\n")
         self.stream.write(
-            f"\nRan {self.testsRun} test{plural} in "
+            f"Ran {self.testsRun} test{plural} in "
             f"{self._delta_to_float(stop - self.__start, 3):.3f}s\n"
         )
         if self.wasSuccessful():
@@ -1245,7 +1344,7 @@ class TextTestResult(TestResult):
             self.stream.write("FAILED (")
             details = []
             failure_count = sum(
-                map(len, (self.failures, self.errors, self.unexpectedSuccesses))
+                len(x) for x in (self.failures, self.errors, self.unexpectedSuccesses)
             )
             details.append(f"failures={failure_count}")
             self.stream.write(", ".join(details))
@@ -1289,8 +1388,8 @@ class ThreadsafeForwardingResult(TestResult):
         self.result = ExtendedToOriginalDecorator(target)
         self.semaphore = semaphore
         self._test_start = None
-        self._global_tags = set(), set()
-        self._test_tags = set(), set()
+        self._global_tags: tuple[set, set] = set(), set()
+        self._test_tags: tuple[set, set] = set(), set()
 
     def __repr__(self):
         return f"<{self.__class__.__name__} {self.result!r}>"
@@ -1507,12 +1606,13 @@ class ExtendedToOriginalDecorator:
                     test.fail("")
                 except test.failureException:
                     return self.addFailure(test, sys.exc_info())
-            if details is not None:
-                try:
-                    return outcome(test, details=details)
-                except TypeError:
-                    pass
-            return outcome(test)
+            else:
+                if details is not None:
+                    try:
+                        return outcome(test, details=details)
+                    except TypeError:
+                        pass
+                return outcome(test)
         finally:
             if self.failfast:
                 self.stop()
@@ -1601,7 +1701,7 @@ class ExtendedToOriginalDecorator:
 
     def stopTest(self, test):
         # NOTE: In Python 3.12.1 skipped tests may not call startTest()
-        if self._tags is not None:
+        if self._tags is not None and self._tags.parent is not None:
             self._tags = self._tags.parent
         return self.decorated.stopTest(test)
 
@@ -1643,6 +1743,7 @@ class ExtendedToStreamDecorator(CopyStreamResult, StreamSummary, TestControl):
         # Deal with mismatched base class constructors.
         TestControl.__init__(self)
         self._started = False
+        self._tags: TagContext | None = None
 
     def _get_failfast(self):
         return len(self.targets) == 2
@@ -1757,6 +1858,8 @@ class ExtendedToStreamDecorator(CopyStreamResult, StreamSummary, TestControl):
     @property
     def current_tags(self):
         """The currently set tags."""
+        if self._tags is None:
+            return set()
         return self._tags.get_current_tags()
 
     def tags(self, new_tags, gone_tags):
@@ -1765,7 +1868,8 @@ class ExtendedToStreamDecorator(CopyStreamResult, StreamSummary, TestControl):
         :param new_tags: A set of tags to be added to the stream.
         :param gone_tags: A set of tags to be removed from the stream.
         """
-        self._tags.change_tags(new_tags, gone_tags)
+        if self._tags is not None:
+            self._tags.change_tags(new_tags, gone_tags)
 
     def _now(self):
         """Return the current 'test time'.
@@ -1862,10 +1966,33 @@ class StreamToExtendedDecorator(StreamResult):
         # _StreamToTestRecord buffers and gives us individual tests.
         self.hook = _StreamToTestRecord(self._handle_tests)
 
-    def status(self, test_id=None, test_status=None, *args, **kwargs):
+    def status(
+        self,
+        test_id=None,
+        test_status=None,
+        test_tags=None,
+        runnable=True,
+        file_name=None,
+        file_bytes=None,
+        eof=False,
+        mime_type=None,
+        route_code=None,
+        timestamp=None,
+    ):
         if test_status == "exists":
             return
-        self.hook.status(test_id=test_id, test_status=test_status, *args, **kwargs)
+        self.hook.status(
+            test_id=test_id,
+            test_status=test_status,
+            test_tags=test_tags,
+            runnable=runnable,
+            file_name=file_name,
+            file_bytes=file_bytes,
+            eof=eof,
+            mime_type=mime_type,
+            route_code=route_code,
+            timestamp=timestamp,
+        )
 
     def startTestRun(self):
         self.decorated.startTestRun()
@@ -1879,6 +2006,36 @@ class StreamToExtendedDecorator(StreamResult):
         case = test_record.to_test_case()
         case.run(self.decorated)
 
+    def wasSuccessful(self):
+        """Return whether this result was successful.
+
+        Delegates to the decorated result object.
+        """
+        return self.decorated.wasSuccessful()
+
+    @property
+    def shouldStop(self):
+        """Return whether the test run should stop.
+
+        Delegates to the decorated result object.
+        """
+        return self.decorated.shouldStop
+
+    def stop(self):
+        """Indicate that the test run should stop.
+
+        Delegates to the decorated result object.
+        """
+        return self.decorated.stop()
+
+    @property
+    def testsRun(self):
+        """Return the number of tests run.
+
+        Delegates to the decorated result object.
+        """
+        return self.decorated.testsRun
+
 
 class StreamToQueue(StreamResult):
     """A StreamResult which enqueues events as a dict to a queue.Queue.
