@@ -18,25 +18,25 @@ from testtools.tests.matchers.helpers import TestMatchersInterface
 
 
 class TestMatchesAllDictInterface(TestCase, TestMatchersInterface):
-    matches_matcher = MatchesAllDict({"a": NotEquals(1), "b": NotEquals(2)})
-    matches_matches: ClassVar[list] = [3, 4]
-    matches_mismatches: ClassVar[list] = [1, 2]
+    matches_matcher: ClassVar = MatchesAllDict({"a": NotEquals(1), "b": NotEquals(2)})
+    matches_matches: ClassVar = [3, 4]
+    matches_mismatches: ClassVar = [1, 2]
 
-    str_examples: ClassVar[list] = [
+    str_examples: ClassVar = [
         ("MatchesAllDict({'a': NotEquals(1), 'b': NotEquals(2)})", matches_matcher)
     ]
 
-    describe_examples: ClassVar[list] = [
+    describe_examples: ClassVar = [
         ("""a: 1 == 1""", 1, matches_matcher),
     ]
 
 
 class TestKeysEqualEmpty(TestCase, TestMatchersInterface):
-    matches_matcher = KeysEqual()
+    matches_matcher: ClassVar = KeysEqual()
     matches_matches: ClassVar[list] = [
         {},
     ]
-    matches_mismatches: ClassVar[list] = [
+    matches_mismatches: ClassVar = [
         {"foo": 0, "bar": 1},
         {"foo": 0},
         {"bar": 1},
@@ -44,21 +44,21 @@ class TestKeysEqualEmpty(TestCase, TestMatchersInterface):
         {"a": None, "b": None, "c": None},
     ]
 
-    str_examples: ClassVar[list] = [
+    str_examples: ClassVar = [
         ("KeysEqual()", KeysEqual()),
     ]
 
-    describe_examples: ClassVar[list] = [
+    describe_examples: ClassVar = [
         ("[] does not match {1: 2}: Keys not equal", {1: 2}, matches_matcher),
     ]
 
 
 class TestKeysEqualWithList(TestCase, TestMatchersInterface):
-    matches_matcher = KeysEqual("foo", "bar")
-    matches_matches: ClassVar[list] = [
+    matches_matcher: ClassVar = KeysEqual("foo", "bar")
+    matches_matches: ClassVar = [
         {"foo": 0, "bar": 1},
     ]
-    matches_mismatches: ClassVar[list] = [
+    matches_mismatches: ClassVar = [
         {},
         {"foo": 0},
         {"bar": 1},
@@ -66,7 +66,7 @@ class TestKeysEqualWithList(TestCase, TestMatchersInterface):
         {"a": None, "b": None, "c": None},
     ]
 
-    str_examples: ClassVar[list] = [
+    str_examples: ClassVar = [
         ("KeysEqual('foo', 'bar')", KeysEqual("foo", "bar")),
     ]
 
@@ -83,18 +83,18 @@ class TestKeysEqualWithList(TestCase, TestMatchersInterface):
 
 
 class TestKeysEqualWithDict(TestKeysEqualWithList):
-    matches_matcher = KeysEqual({"foo": 3, "bar": 4})
+    matches_matcher: ClassVar = KeysEqual({"foo": 3, "bar": 4})
 
 
 class TestSubDictOf(TestCase, TestMatchersInterface):
-    matches_matcher = _SubDictOf({"foo": "bar", "baz": "qux"})
+    matches_matcher: ClassVar = _SubDictOf({"foo": "bar", "baz": "qux"})
 
-    matches_matches: ClassVar[list] = [
+    matches_matches: ClassVar = [
         {"foo": "bar", "baz": "qux"},
         {"foo": "bar"},
     ]
 
-    matches_mismatches: ClassVar[list] = [
+    matches_mismatches: ClassVar = [
         {"foo": "bar", "baz": "qux", "cat": "dog"},
         {"foo": "bar", "cat": "dog"},
     ]
@@ -104,13 +104,15 @@ class TestSubDictOf(TestCase, TestMatchersInterface):
 
 
 class TestMatchesDict(TestCase, TestMatchersInterface):
-    matches_matcher = MatchesDict({"foo": Equals("bar"), "baz": Not(Equals("qux"))})
+    matches_matcher: ClassVar = MatchesDict(
+        {"foo": Equals("bar"), "baz": Not(Equals("qux"))}
+    )
 
-    matches_matches: ClassVar[list] = [
+    matches_matches: ClassVar = [
         {"foo": "bar", "baz": None},
         {"foo": "bar", "baz": "quux"},
     ]
-    matches_mismatches: ClassVar[list] = [
+    matches_mismatches: ClassVar = [
         {},
         {"foo": "bar", "baz": "qux"},
         {"foo": "bop", "baz": "qux"},
@@ -118,7 +120,7 @@ class TestMatchesDict(TestCase, TestMatchersInterface):
         {"foo": "bar", "cat": "dog"},
     ]
 
-    str_examples: ClassVar[list] = [
+    str_examples: ClassVar = [
         (
             "MatchesDict({{'baz': {}, 'foo': {}}})".format(
                 Not(Equals("qux")), Equals("bar")
@@ -127,7 +129,7 @@ class TestMatchesDict(TestCase, TestMatchersInterface):
         ),
     ]
 
-    describe_examples: ClassVar[list] = [
+    describe_examples: ClassVar = [
         (
             "Missing: {\n  'baz': Not(Equals('qux')),\n  'foo': Equals('bar'),\n}",
             {},
@@ -160,14 +162,16 @@ class TestMatchesDict(TestCase, TestMatchersInterface):
 
 
 class TestContainsDict(TestCase, TestMatchersInterface):
-    matches_matcher = ContainsDict({"foo": Equals("bar"), "baz": Not(Equals("qux"))})
+    matches_matcher: ClassVar = ContainsDict(
+        {"foo": Equals("bar"), "baz": Not(Equals("qux"))}
+    )
 
-    matches_matches: ClassVar[list] = [
+    matches_matches: ClassVar = [
         {"foo": "bar", "baz": None},
         {"foo": "bar", "baz": "quux"},
         {"foo": "bar", "baz": "quux", "cat": "dog"},
     ]
-    matches_mismatches: ClassVar[list] = [
+    matches_mismatches: ClassVar = [
         {},
         {"foo": "bar", "baz": "qux"},
         {"foo": "bop", "baz": "qux"},
@@ -175,7 +179,7 @@ class TestContainsDict(TestCase, TestMatchersInterface):
         {"foo": "bar"},
     ]
 
-    str_examples: ClassVar[list] = [
+    str_examples: ClassVar = [
         (
             "ContainsDict({{'baz': {}, 'foo': {}}})".format(
                 Not(Equals("qux")), Equals("bar")
@@ -184,7 +188,7 @@ class TestContainsDict(TestCase, TestMatchersInterface):
         ),
     ]
 
-    describe_examples: ClassVar[list] = [
+    describe_examples: ClassVar = [
         (
             "Missing: {\n  'baz': Not(Equals('qux')),\n  'foo': Equals('bar'),\n}",
             {},
@@ -212,22 +216,24 @@ class TestContainsDict(TestCase, TestMatchersInterface):
 
 
 class TestContainedByDict(TestCase, TestMatchersInterface):
-    matches_matcher = ContainedByDict({"foo": Equals("bar"), "baz": Not(Equals("qux"))})
+    matches_matcher: ClassVar = ContainedByDict(
+        {"foo": Equals("bar"), "baz": Not(Equals("qux"))}
+    )
 
-    matches_matches: ClassVar[list] = [
+    matches_matches: ClassVar = [
         {},
         {"foo": "bar"},
         {"foo": "bar", "baz": "quux"},
         {"baz": "quux"},
     ]
-    matches_mismatches: ClassVar[list] = [
+    matches_mismatches: ClassVar = [
         {"foo": "bar", "baz": "quux", "cat": "dog"},
         {"foo": "bar", "baz": "qux"},
         {"foo": "bop", "baz": "qux"},
         {"foo": "bar", "cat": "dog"},
     ]
 
-    str_examples: ClassVar[list] = [
+    str_examples: ClassVar = [
         (
             "ContainedByDict({{'baz': {}, 'foo': {}}})".format(
                 Not(Equals("qux")), Equals("bar")
@@ -236,7 +242,7 @@ class TestContainedByDict(TestCase, TestMatchersInterface):
         ),
     ]
 
-    describe_examples: ClassVar[list] = [
+    describe_examples: ClassVar = [
         (
             "Differences: {\n  'baz': 'qux' matches Equals('qux'),\n}",
             {"foo": "bar", "baz": "qux"},
