@@ -4,7 +4,7 @@
 
 import os
 import signal
-from typing import ClassVar
+from typing import Any, ClassVar
 
 from testtools import (
     TestCase,
@@ -35,7 +35,7 @@ from ._helpers import NeedsTwistedTestCase
 try:
     from testtools.twistedsupport._deferreddebug import DebugTwisted
 except ImportError:
-    DebugTwisted = None
+    DebugTwisted = None  # type: ignore[assignment,misc]
 
 try:
     from testtools.twistedsupport import (
@@ -45,31 +45,31 @@ try:
         flush_logged_errors,
     )
 except ImportError:
-    assert_fails_with = None
-    AsynchronousDeferredRunTest = None
-    flush_logged_errors = None
-    SynchronousDeferredRunTest = None
+    assert_fails_with = None  # type: ignore[assignment,misc]
+    AsynchronousDeferredRunTest = None  # type: ignore[assignment,misc]
+    flush_logged_errors = None  # type: ignore[assignment,misc]
+    SynchronousDeferredRunTest = None  # type: ignore[assignment,misc]
 
 try:
     from twisted.internet import defer
 except ImportError:
-    defer = None
+    defer = None  # type: ignore[assignment,misc]
 
 try:
     from twisted.python import failure, log
 except ImportError:
-    failure = None
-    log = None
+    failure = None  # type: ignore[assignment,misc]
+    log = None  # type: ignore[assignment,misc]
 
 try:
     from twisted.internet.base import DelayedCall
 except ImportError:
-    DelayedCall = None
+    DelayedCall = None  # type: ignore[assignment,misc]
 
 try:
     from testtools.twistedsupport._runtest import _get_global_publisher_and_observers
 except ImportError:
-    _get_global_publisher_and_observers = None
+    _get_global_publisher_and_observers = None  # type: ignore[assignment,misc]
 
 
 class X:
@@ -146,6 +146,10 @@ class X:
             self.expectThat(object(), Is(object()))
 
     class TestIntegration(NeedsTwistedTestCase):
+        # These attributes are set dynamically in test generation
+        test_factory: Any = None
+        runner: Any = None
+
         def assertResultsMatch(self, test, result):
             events = list(result._events)
             self.assertEqual(("startTest", test), events.pop(0))
@@ -287,9 +291,9 @@ class TestAsynchronousDeferredRunTest(NeedsTwistedTestCase):
         # setUp can return a Deferred that might fire at any time.
         # AsynchronousDeferredRunTest will not go on to running the test until
         # the Deferred returned by setUp actually fires.
-        call_log = []
+        call_log: list[Any] = []
         marker = object()
-        d = defer.Deferred().addCallback(call_log.append)
+        d: defer.Deferred[Any] = defer.Deferred().addCallback(call_log.append)
 
         class SomeCase(TestCase):
             def setUp(self):
@@ -302,7 +306,7 @@ class TestAsynchronousDeferredRunTest(NeedsTwistedTestCase):
 
         def fire_deferred():
             self.assertThat(call_log, Equals(["setUp"]))
-            d.callback(marker)
+            d.callback(marker)  # type: ignore[arg-type]
 
         test = SomeCase("test_something")
         timeout = self.make_timeout()
@@ -318,12 +322,12 @@ class TestAsynchronousDeferredRunTest(NeedsTwistedTestCase):
         # Deferreds. AsynchronousDeferredRunTest will make sure that each of
         # these are run in turn, only going on to the next stage once the
         # Deferred from the previous stage has fired.
-        call_log = []
-        a = defer.Deferred()
+        call_log: list[Any] = []
+        a: defer.Deferred[Any] = defer.Deferred()
         a.addCallback(lambda x: call_log.append("a"))
-        b = defer.Deferred()
+        b: defer.Deferred[Any] = defer.Deferred()
         b.addCallback(lambda x: call_log.append("b"))
-        c = defer.Deferred()
+        c: defer.Deferred[Any] = defer.Deferred()
         c.addCallback(lambda x: call_log.append("c"))
 
         class SomeCase(TestCase):
@@ -374,7 +378,7 @@ class TestAsynchronousDeferredRunTest(NeedsTwistedTestCase):
                 pass
 
         test = SomeCase("test_whatever")
-        call_log = []
+        call_log: list[Any] = []
         a = defer.Deferred().addCallback(lambda x: call_log.append("a"))
         b = defer.Deferred().addCallback(lambda x: call_log.append("b"))
         c = defer.Deferred().addCallback(lambda x: call_log.append("c"))
@@ -431,7 +435,7 @@ class TestAsynchronousDeferredRunTest(NeedsTwistedTestCase):
 
         class SomeCase(TestCase):
             def test_cruft(self):
-                self.assertIs(reactor, self.reactor)
+                self.assertIs(reactor, self.reactor)  # type: ignore[attr-defined]
 
         test = SomeCase("test_cruft")
         runner = self.make_runner(test, timeout)
@@ -780,7 +784,7 @@ class TestAsynchronousDeferredRunTest(NeedsTwistedTestCase):
     def test_do_not_log_to_twisted(self):
         # If suppress_twisted_logging is True, we don't log anything to the
         # default Twisted loggers.
-        messages = []
+        messages: list[Any] = []
         publisher, _ = _get_global_publisher_and_observers()
         publisher.addObserver(messages.append)
         self.addCleanup(publisher.removeObserver, messages.append)
@@ -798,7 +802,7 @@ class TestAsynchronousDeferredRunTest(NeedsTwistedTestCase):
     def test_log_to_twisted(self):
         # If suppress_twisted_logging is False, we log to the default Twisted
         # loggers.
-        messages = []
+        messages: list[Any] = []
         publisher, _ = _get_global_publisher_and_observers()
         publisher.addObserver(messages.append)
 
@@ -987,7 +991,7 @@ class TestNoTwistedLogObservers(NeedsTwistedTestCase):
 
     def _get_logged_messages(self, function, *args, **kwargs):
         """Run ``function`` and return ``(ret, logged_messages)``."""
-        messages = []
+        messages: list[Any] = []
         publisher, _ = _get_global_publisher_and_observers()
         publisher.addObserver(messages.append)
         try:
@@ -1048,7 +1052,7 @@ class TestTwistedLogObservers(NeedsTwistedTestCase):
         # that observer while the fixture is active.
         from testtools.twistedsupport._runtest import _TwistedLogObservers
 
-        messages = []
+        messages: list[Any] = []
 
         class SomeTest(TestCase):
             def test_something(self):
